from data_amqp.amqp import AMQPData


amqp_options = {
    'code':'gnss',
    'vhost':'gpsdata',
    'exchange': 'exch_data',
    'host': '10.54.218.158',
    'credentials': ('gps','=0GPS0='),
    'qname':'gpsstream',
    'durable':True
}

value = None

mq = AMQPData(**amqp_options)

while not value == '-1':
    value = input("Entregame un mensaje para la cola gpsstream\n")
    enviar = {'msg': value}
    mq.send2gui(enviar)




#!/usr/bin/env python
from data_amqp.amqp import AMQPData
from tasktools.taskloop import TaskLoop
import asyncio
from networktools.colorprint import gprint, rprint


amqp_options = {
    'code': 'test',
    'vhost': '/gendata',
    'exchange': 'gendata',
    'host': 'localhost',
    "port": 5672,
    'credentials': ('recvdata', 'recvdata'),
    'queue_name': 'stations',
    'durable': True,
    "limit": 20,
    "routing_key": "gendata_stations",
    "consumer_tag": "gendata"
}

value = None

mq = AMQPData(**amqp_options)
mq.connect()
queue = asyncio.Queue()

from pprint import pprint

async def consume(*args, **kwargs):
    amqp = args[0]
    queue = args[1]
    print(amqp.channel.consume)
    count = 0
    for method_frame, properties, body in amqp.channel.consume('stations'):
        gprint(f"{method_frame}->{body}")
        amqp.channel.basic_ack(method_frame.delivery_tag)
        await queue.put(body)
        count += 1
        if count >= 1:
            break
    return args, kwargs

async def basic_consume(*args, **kwargs):
    print("Basic_consume")
    await asyncio.sleep(2)
    mq.channel.basic_consume("stations", print)
    return args, kwargs

async def read_queue(*args, **kwargs):
    queue = args[0]
    print("Leer cola", queue)
    await asyncio.sleep(.5)
    if not queue.empty():
        await queue.join()
        for i in range(queue.qsize()):
            elem = await queue.get()
            pprint(elem)
            queue.task_done()
    return [queue, *args], kwargs


loop = asyncio.get_event_loop()
task_a = TaskLoop(consume, [mq, queue], {})
task_a.create()
task_b = TaskLoop(read_queue, [queue], {})
task_b.create()
print(task_b)
loop.run_forever()



from data_amqp.amqp import AMQPData

amqp_options = {
    'code':'gnss',
    'vhost':'/gendata',
    'exchange': 'gendata',
    'host': 'localhost',
    'port': 5672,
    'credentials': ('gendata','gendata'),
    'queue_name':'stations',
    "routing_key":"gendata_stations",
    'durable':True
}

value = None

mq = AMQPData(**amqp_options)

while not value == '-1':
    value = input("Entregame un mensaje para la cola gpsstream\n")
    enviar = {'msg': value}
    mq.send2gui(enviar)




from networktools.geo import (rad2deg, deg2rad, radius, sph_rotation_matrix,
                              llh2ecef, ecef2llh, ecef2neu)
from networktools.time import timestamp, gps_week2time, gps_time
from networktools.statistics import matrix_VCV, rotate
from data_geo.geojson import GeoJSONData
import json
import pika
import copy
import queue
# from networktools.colorprint import gprint, bprint, rprint
import asyncio
from datadbs.general import GeneralData
from tasktools.taskloop import TaskLoop
from pprint import pprint


class AMQPData(GeneralData):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.creds = kwargs.get('credentials', [])
        self.vhost = kwargs.get('vhost', '/')
        self.exchange = kwargs.get('exchange', '')
        self.exch_type = kwargs.get('exchange_type', 'fanout')
        self.queue_name = kwargs.get('queue_name', 'status_gnss')
        self.routing_key = kwargs.get('routing_key', 'status')
        self.consumer_tag = kwargs.get('consumer_tag', 'gnss')
        self.durable = kwargs.get('durable', True)
        self.limit = kwargs.get('limit', 5)
        self.ping = kwargs.get('ping', 30)
        self.multi = kwargs.get('multiprocessing', False)
        self.ttl = kwargs.get('ttl',1200000)
        self.source = kwargs.get('source',False)
        try:
            self.connect()
        except Exception as e:
            self.logger.exception(
                "Error en conexión Rabbitmq. %s, args %s, kwargs %s" %
                (e, args, kwargs))

    def connect(self):
        try:
            self.credentials = pika.PlainCredentials(*self.creds)
        except Exception as e:
            self.logger.exception("Error creando credenciales Rabbitmq")
            raise e
        try:
            self.parameters = pika.ConnectionParameters(
                host=self.host,
                virtual_host=self.vhost,
                credentials=self.credentials,
                heartbeat=self.ping)
        except Exception as e:
            self.logger.exception("Error conectando parámetros Rabbitmq")
            raise e
        try:
            self.connection = pika.BlockingConnection(self.parameters)
        except Exception as e:
            self.logger.exception(
                "Error conectando Rabbitmq, bloquing connection")
            raise e
        try:
            self.channel = self.connection.channel()
            # bind by default to:
            # conecta exchange con queue
            self.connected = True
        except Exception as e:
            self.logger.exception("Error conectando canal Rabbitmq")
            raise e
        try:
            if self.source:
                self.channel.queue_declare(self.queue_name, 
                    durable=self.durable, 
                    arguments={'x-message-ttl' : self.ttl})
                self.channel.queue_bind(self.queue_name, self.exchange, self.routing_key)
            else:
                self.queue_name = self.channel.queue_declare(exclusive=True)[0]
                self.channel.queue_bind(self.queue_name, exchange=self.exchange)
        except Exception as e:
            self.logger.exception("Error declaring queue -> %s" % e)
            self.connected = False
            raise e

    def reconnect(self):
        """Will be invoked if the connection can't be opened or is
        closed. Indicates that a reconnect is necessary then stops the
        ioloop.
        """
        self.should_reconnect = True
        self.stop()

    def manage_data(self, geojson):
        value = copy.deepcopy(geojson)
        if 'DT_GEN' in value['properties'].keys():
            del value['properties']['DT_GEN']
        if 'dt' in value['properties'].keys():
            del value['properties']['dt']
        text = json.dumps(value)
        try:
            res = self.publish(self.exchange, self.routing_key,
                               "{}\n".format(text))
        except Exception as e:
            self.logger.exception("Error publicando datos -> %s, text" %
                                  (e, text))
        del value
        return geojson

    def manage_json_data(self, value):
        text = json.dumps(value)
        print("Exchange", self.exchange, self.routing_key)
        res = self.publish(self.exchange, self.routing_key,
                           f"{text}\n")
        return value

    def send2gui(self, to_gui):
        try:
            text = json.dumps(to_gui)
            print("Exchange", self.exchange, self.routing_key)
            res = self.publish(self.exchange, self.routing_key,
                               f"{to_gui}\n")
            return res
        except Exception as e:
            self.logger("Exception en send a gui %s" % e)
            raise e

    def publish(self, exchange, key, msg):
        sended = False
        while not sended:
            try:
                self.channel.basic_publish(exchange=exchange,
                                           routing_key=self.routing_key,
                                           body=msg)
                sended = True
            except Exception as e:
                self.connected = False
                print("Fail to send %s" % e)
                self.connect()

    def read_from_gui(self):
        data = []
        count = 0
        for method_frame, properties, body in self.channel.consume(
                self.queue_name):
            try:
                strb = str(body, 'utf-8').replace("'", "\"")
                new_json = json.loads(strb)
            except Exception as e:
                self.logger.exception("Error en leer desde gui %s" % e)
                raise e
            data.append(new_json)
            self.channel.basic_ack(method_frame.delivery_tag)
            count += 1
            if count >= 24:
                break
        return data

    async def consume_mq(self, *args, **kwargs):
        queue = args[0]
        active = args[1]
        count = 0
        consumer_args = self.get_consumer_args()
        for method_frame, properties, body in self.channel.consume(
                *consumer_args):
            try:
                strb = str(body, 'utf-8').replace("'", "\"")
                new_json = json.loads(strb)
            except Exception as e:
                self.logger.exception(
                    "Error en transformar json, consume_mq, error %s" % e)
                raise e
            if active:
                if isinstance(queue, queue.Queue):
                    queue.put(new_json)
                elif isinstance(queue, asyncio.queue.Queue):
                    await queue.put(new_json)
                elif self.multi:
                    queue.put(new_json)
            else:
                self.logger.info("Consumeindo json %s" % json)
            # self.channel.basic_ack(method_frame.delivery_tag)
            count += 1
            if count >= 24:
                break
        return args, kwargs

    def get_consumer_args(self):
        return [self.queue_name, self.consumer_tag]


    async def consume_exchange_mq(self, *args, **kwargs):
        queue_destiny = args[0]
        active = args[1]
        count = 0
        consumer_args = self.get_consumer_args()
        print("Consuming from exchange", consumer_args)
        for method_frame, properties, body in self.channel.consume(
                *consumer_args):
            try:
                strb = str(body, 'utf-8').replace("'", "\"")
                new_json = json.loads(strb)
            except Exception as e:
                self.logger.exception(
                    "Error on consume_exchange_mq convert json %s, error %e" %
                    (body, e))
                raise e
            if active:
                if isinstance(queue_destiny, queue.Queue):
                    queue_destiny.put(new_json)
                elif isinstance(queue_destiny, asyncio.Queue):
                    await queue_destiny.put(new_json)
                elif self.multi:
                    queue_destiny.put(new_json)
                else:
                    print("No enviado")
            else:
                self.logger.info("Consumeindo json %s" % json)
            count += 1
            if count >= self.limit:
                break
        return args, kwargs

    def create_task(self, queue_mq2ws, active):
        try:
            loop = asyncio.get_event_loop()
            args = [queue_mq2ws, active]
            kwargs = {}
            task = TaskLoop(self.consume_mq, args, kwargs)
            task.create()
        except Exception as e:
            self.logger.exception("Error on create_task RMQ %s, error %e" %
                                  (queue_mq2ws, e))
        return loop, task

    def create_exchange_task(self, queue_mq2ws, active):
        try:
            loop = asyncio.get_event_loop()
            args = [queue_mq2ws, active]
            kwargs = {}
            task = TaskLoop(self.consume_exchange_mq, args, kwargs)
            print("Tarea consumir data", task)
            task.create()
        except Exception as e:
            self.logger.exception(
                "Error on create_exchange_task RMQ %s, error %e" %
                (queue_mq2ws, e))
        return loop, task

    def consume_mq_task(self, queue_mq2ws, active):
        loop, task = self.create_task(queue_mq2ws, active)
        if not loop.is_running():
            loop.run_forever()



from setuptools import setup, find_packages

from pathlib import Path

path = Path(__file__).resolve().parent

with open(path/'README.md', encoding='utf-8') as f:
    long_description = f.read()

with open(path/'VERSION') as version_file:
    version = version_file.read().strip()

setup(name='data_amqp',
      version=version,
      description='DataDBS for RabbitMQP',
      url='http://www.gitlab.com/dpineda/data_amqp',
      author='David Pineda Osorio',
      author_email='dpineda@csn.uchile.cl',
      license='GPLv3',
      packages=["data_amqp"],
      install_requires=["networktools","data_geo","pika","datadbs", "tasktools"],
      package_dir={'data_amqp': 'data_amqp'},
      package_data={
          'data_amqp': ['../doc', '../docs', '../requeriments.txt']},
      long_description=long_description,
      long_description_content_type='text/markdown',
      zip_safe=False)
